/* Test fromfp.
   Copyright (C) 1997-2020 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */

#include "libm-test-driver.c"

static const struct test_fiu_M_data fromfp_test_data[] =
  {
    /* Infinity, NaN and values that are definitely out of range
       (large values of width count as the width of the return
       type).  */
    TEST_fiu_M (fromfp, plus_infty, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_infty, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_infty, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, qnan_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -qnan_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, snan_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -snan_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, max_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -max_value, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1p64, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1p64, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p63, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p63, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000001p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif

    /* Zero (OK unless the width specified is zero).  */
    TEST_fiu_M (fromfp, plus_zero, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, plus_zero, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, minus_zero, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),

    /* Tests below here are generated with gen-fromfp-tests.py.  */

    /* Integer inputs within range of uintmax_t.  */
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_DOWNWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TOWARDZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_UPWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TOWARDZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_UPWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_DOWNWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TOWARDZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5, FP_INT_TONEAREST, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_UPWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_DOWNWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TOWARDZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5, FP_INT_TONEAREST, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_UPWARD, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_DOWNWARD, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TOWARDZERO, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEARESTFROMZERO, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 6, FP_INT_TONEAREST, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_UPWARD, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_DOWNWARD, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TOWARDZERO, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEARESTFROMZERO, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -6, FP_INT_TONEAREST, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 4, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 5, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, 64, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_UPWARD, UINT_MAX, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 4, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 5, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, 64, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_DOWNWARD, UINT_MAX, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 4, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 5, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, 64, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TOWARDZERO, UINT_MAX, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 4, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 5, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, 64, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEARESTFROMZERO, UINT_MAX, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 4, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 5, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, 64, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 7, FP_INT_TONEAREST, UINT_MAX, 7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 4, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 5, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, 64, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_UPWARD, UINT_MAX, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 4, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 5, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, 64, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_DOWNWARD, UINT_MAX, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 4, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 5, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, 64, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TOWARDZERO, UINT_MAX, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 4, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 5, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, 64, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEARESTFROMZERO, UINT_MAX, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 4, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 5, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, 64, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -7, FP_INT_TONEAREST, UINT_MAX, -7, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 5, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 6, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, 64, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_UPWARD, UINT_MAX, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 5, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 6, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, 64, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_DOWNWARD, UINT_MAX, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 5, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 6, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, 64, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TOWARDZERO, UINT_MAX, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 5, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 6, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, 64, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEARESTFROMZERO, UINT_MAX, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 5, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 6, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, 64, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 8, FP_INT_TONEAREST, UINT_MAX, 8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 4, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 5, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, 64, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_UPWARD, UINT_MAX, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 4, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 5, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, 64, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_DOWNWARD, UINT_MAX, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 4, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 5, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, 64, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TOWARDZERO, UINT_MAX, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 4, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 5, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, 64, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEARESTFROMZERO, UINT_MAX, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 4, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 5, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, 64, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -8, FP_INT_TONEAREST, UINT_MAX, -8, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 5, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 6, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, 64, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_UPWARD, UINT_MAX, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 5, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 6, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, 64, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_DOWNWARD, UINT_MAX, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 5, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 6, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, 64, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TOWARDZERO, UINT_MAX, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 5, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 6, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, 64, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEARESTFROMZERO, UINT_MAX, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 5, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 6, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, 64, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 9, FP_INT_TONEAREST, UINT_MAX, 9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 5, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 6, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, 64, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_UPWARD, UINT_MAX, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 5, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 6, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, 64, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_DOWNWARD, UINT_MAX, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 5, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 6, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, 64, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TOWARDZERO, UINT_MAX, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 5, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 6, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, 64, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEARESTFROMZERO, UINT_MAX, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 5, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 6, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, 64, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -9, FP_INT_TONEAREST, UINT_MAX, -9, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 5, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 6, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, 64, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_UPWARD, UINT_MAX, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 5, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 6, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, 64, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_DOWNWARD, UINT_MAX, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 5, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 6, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, 64, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TOWARDZERO, UINT_MAX, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 5, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 6, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, 64, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEARESTFROMZERO, UINT_MAX, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 5, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 6, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, 64, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 15, FP_INT_TONEAREST, UINT_MAX, 15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 5, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 6, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, 64, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_UPWARD, UINT_MAX, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 5, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 6, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, 64, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_DOWNWARD, UINT_MAX, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 5, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 6, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, 64, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TOWARDZERO, UINT_MAX, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 5, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 6, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, 64, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEARESTFROMZERO, UINT_MAX, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 5, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 6, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, 64, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -15, FP_INT_TONEAREST, UINT_MAX, -15, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 6, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 7, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, 64, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_UPWARD, UINT_MAX, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 6, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 7, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, 64, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_DOWNWARD, UINT_MAX, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 6, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 7, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, 64, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TOWARDZERO, UINT_MAX, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 6, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 7, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, 64, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEARESTFROMZERO, UINT_MAX, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 6, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 7, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, 64, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 16, FP_INT_TONEAREST, UINT_MAX, 16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 5, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 6, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, 64, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_UPWARD, UINT_MAX, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 5, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 6, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, 64, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_DOWNWARD, UINT_MAX, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 5, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 6, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, 64, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TOWARDZERO, UINT_MAX, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 5, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 6, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, 64, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEARESTFROMZERO, UINT_MAX, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 4, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 5, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 6, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, 64, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -16, FP_INT_TONEAREST, UINT_MAX, -16, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 6, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 7, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, 64, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_UPWARD, UINT_MAX, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 6, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 7, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, 64, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_DOWNWARD, UINT_MAX, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 6, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 7, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, 64, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TOWARDZERO, UINT_MAX, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 6, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 7, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, 64, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEARESTFROMZERO, UINT_MAX, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 6, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 7, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, 64, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 17, FP_INT_TONEAREST, UINT_MAX, 17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 6, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 7, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, 64, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_UPWARD, UINT_MAX, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 6, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 7, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, 64, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_DOWNWARD, UINT_MAX, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 6, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 7, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, 64, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TOWARDZERO, UINT_MAX, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 6, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 7, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, 64, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEARESTFROMZERO, UINT_MAX, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 5, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 6, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 7, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, 64, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -17, FP_INT_TONEAREST, UINT_MAX, -17, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_UPWARD, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_DOWNWARD, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TOWARDZERO, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff, FP_INT_TONEAREST, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_UPWARD, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_DOWNWARD, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TOWARDZERO, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff, FP_INT_TONEAREST, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_UPWARD, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_DOWNWARD, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TOWARDZERO, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400000, FP_INT_TONEAREST, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_UPWARD, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_DOWNWARD, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TOWARDZERO, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400000, FP_INT_TONEAREST, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 24, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 25, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, 64, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_UPWARD, UINT_MAX, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 24, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 25, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, 64, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_DOWNWARD, UINT_MAX, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 24, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 25, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, 64, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TOWARDZERO, UINT_MAX, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 24, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 25, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, 64, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 24, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 25, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, 64, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x400001, FP_INT_TONEAREST, UINT_MAX, 0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 24, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 25, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, 64, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_UPWARD, UINT_MAX, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 24, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 25, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, 64, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_DOWNWARD, UINT_MAX, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 24, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 25, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, 64, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TOWARDZERO, UINT_MAX, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 24, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 25, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, 64, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 24, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 25, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, 64, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x400001, FP_INT_TONEAREST, UINT_MAX, -0x400001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 24, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 25, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, 64, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_UPWARD, UINT_MAX, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 24, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 25, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, 64, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_DOWNWARD, UINT_MAX, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 24, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 25, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, 64, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 24, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 25, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, 64, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 24, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 25, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, 64, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff, FP_INT_TONEAREST, UINT_MAX, 0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 24, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 25, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, 64, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_UPWARD, UINT_MAX, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 24, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 25, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, 64, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_DOWNWARD, UINT_MAX, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 24, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 25, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, 64, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TOWARDZERO, UINT_MAX, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 24, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 25, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, 64, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 24, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 25, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, 64, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff, FP_INT_TONEAREST, UINT_MAX, -0x7fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 25, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 26, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, 64, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_UPWARD, UINT_MAX, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 25, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 26, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, 64, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_DOWNWARD, UINT_MAX, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 25, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 26, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, 64, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TOWARDZERO, UINT_MAX, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 25, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 26, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, 64, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 25, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 26, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, 64, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800000, FP_INT_TONEAREST, UINT_MAX, 0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 24, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 25, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, 64, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_UPWARD, UINT_MAX, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 24, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 25, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, 64, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_DOWNWARD, UINT_MAX, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 24, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 25, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, 64, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TOWARDZERO, UINT_MAX, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 24, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 25, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, 64, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 24, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 25, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, 64, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800000, FP_INT_TONEAREST, UINT_MAX, -0x800000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 25, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 26, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, 64, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_UPWARD, UINT_MAX, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 25, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 26, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, 64, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_DOWNWARD, UINT_MAX, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 25, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 26, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, 64, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TOWARDZERO, UINT_MAX, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 25, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 26, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, 64, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 25, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 26, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, 64, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x800001, FP_INT_TONEAREST, UINT_MAX, 0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 25, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 26, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, 64, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_UPWARD, UINT_MAX, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 25, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 26, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, 64, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_DOWNWARD, UINT_MAX, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 25, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 26, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, 64, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TOWARDZERO, UINT_MAX, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 25, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 26, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, 64, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 25, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 26, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, 64, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x800001, FP_INT_TONEAREST, UINT_MAX, -0x800001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 25, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 26, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, 64, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_UPWARD, UINT_MAX, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 25, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 26, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, 64, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_DOWNWARD, UINT_MAX, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 25, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 26, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, 64, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TOWARDZERO, UINT_MAX, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 25, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 26, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, 64, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 25, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 26, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, 64, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffff, FP_INT_TONEAREST, UINT_MAX, 0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 25, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 26, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, 64, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_UPWARD, UINT_MAX, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 25, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 26, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, 64, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_DOWNWARD, UINT_MAX, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 25, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 26, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, 64, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TOWARDZERO, UINT_MAX, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 25, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 26, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, 64, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 25, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 26, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, 64, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffff, FP_INT_TONEAREST, UINT_MAX, -0xffffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 26, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 27, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, 64, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_UPWARD, UINT_MAX, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 26, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 27, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, 64, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_DOWNWARD, UINT_MAX, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 26, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 27, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, 64, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TOWARDZERO, UINT_MAX, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 26, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 27, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, 64, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 26, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 27, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, 64, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000000, FP_INT_TONEAREST, UINT_MAX, 0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 25, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 26, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, 64, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_UPWARD, UINT_MAX, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 25, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 26, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, 64, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_DOWNWARD, UINT_MAX, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 25, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 26, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, 64, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TOWARDZERO, UINT_MAX, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 25, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 26, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, 64, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 24, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 25, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 26, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, 64, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000000, FP_INT_TONEAREST, UINT_MAX, -0x1000000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 26, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 27, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, 64, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_UPWARD, UINT_MAX, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 26, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 27, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, 64, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_DOWNWARD, UINT_MAX, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 26, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 27, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, 64, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TOWARDZERO, UINT_MAX, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 26, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 27, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, 64, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 26, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 27, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, 64, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1000001, FP_INT_TONEAREST, UINT_MAX, 0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 26, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 27, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, 64, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_UPWARD, UINT_MAX, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 26, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 27, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, 64, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_DOWNWARD, UINT_MAX, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 26, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 27, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, 64, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TOWARDZERO, UINT_MAX, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 26, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 27, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, 64, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 25, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 26, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 27, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, 64, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1000001, FP_INT_TONEAREST, UINT_MAX, -0x1000001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 52, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 53, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, 64, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_UPWARD, UINT_MAX, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 52, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 53, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, 64, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 52, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 53, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, 64, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 52, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 52, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 53, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, 64, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, 0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 52, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 53, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, 64, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_UPWARD, UINT_MAX, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 52, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 53, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, 64, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 52, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 53, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, 64, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 52, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 52, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 53, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, 64, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, -0x7ffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 53, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 54, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, 64, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_UPWARD, UINT_MAX, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 53, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 54, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, 64, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_DOWNWARD, UINT_MAX, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 53, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 54, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, 64, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 53, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 54, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 64, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 53, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 54, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, 64, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000LL, FP_INT_TONEAREST, UINT_MAX, 0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 52, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 53, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, 64, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_UPWARD, UINT_MAX, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 52, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 53, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, 64, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_DOWNWARD, UINT_MAX, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 52, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 53, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, 64, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 52, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 53, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, 64, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 51, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 52, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 53, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, 64, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000LL, FP_INT_TONEAREST, UINT_MAX, -0x8000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 53, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 54, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, 64, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_UPWARD, UINT_MAX, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 53, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 54, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, 64, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_DOWNWARD, UINT_MAX, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 53, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 54, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, 64, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 53, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 54, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 64, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 53, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 54, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, 64, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000001LL, FP_INT_TONEAREST, UINT_MAX, 0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 53, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 54, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, 64, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_UPWARD, UINT_MAX, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 53, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 54, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, 64, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_DOWNWARD, UINT_MAX, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 53, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 54, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, 64, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 53, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 54, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, 64, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 53, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 54, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, 64, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000001LL, FP_INT_TONEAREST, UINT_MAX, -0x8000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 53, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 54, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, 64, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_UPWARD, UINT_MAX, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 53, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 54, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, 64, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 53, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 54, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, 64, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 54, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 53, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 54, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, 64, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, 0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 53, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 54, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, 64, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_UPWARD, UINT_MAX, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 53, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 54, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, 64, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 53, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 54, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, 64, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 54, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 53, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 54, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, 64, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, -0xfffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 54, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 55, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, 64, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_UPWARD, UINT_MAX, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 54, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 55, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, 64, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_DOWNWARD, UINT_MAX, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 54, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 55, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, 64, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 54, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 55, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 64, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 54, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 55, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, 64, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000000LL, FP_INT_TONEAREST, UINT_MAX, 0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 53, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 54, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, 64, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_UPWARD, UINT_MAX, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 53, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 54, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, 64, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_DOWNWARD, UINT_MAX, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 53, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 54, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, 64, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 53, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 54, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, 64, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 52, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 53, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 54, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, 64, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000000LL, FP_INT_TONEAREST, UINT_MAX, -0x10000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 54, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 55, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, 64, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_UPWARD, UINT_MAX, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 54, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 55, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, 64, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_DOWNWARD, UINT_MAX, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 54, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 55, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, 64, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 54, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 55, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 64, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 54, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 55, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, 64, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x10000000000001LL, FP_INT_TONEAREST, UINT_MAX, 0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 54, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 55, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, 64, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_UPWARD, UINT_MAX, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 54, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 55, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, 64, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_DOWNWARD, UINT_MAX, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 54, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 55, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, 64, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 54, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 55, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, 64, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 54, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 55, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, 64, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x10000000000001LL, FP_INT_TONEAREST, UINT_MAX, -0x10000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 54, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 55, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, 64, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_UPWARD, UINT_MAX, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 54, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 55, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, 64, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 54, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 55, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, 64, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 54, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 55, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 54, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 55, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, 64, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, 0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 54, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 55, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, 64, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_UPWARD, UINT_MAX, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 54, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 55, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, 64, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 54, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 55, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, 64, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 54, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 55, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 54, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 55, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, 64, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, -0x1fffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 55, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 56, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, 64, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_UPWARD, UINT_MAX, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 55, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 56, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, 64, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_DOWNWARD, UINT_MAX, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 55, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 56, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, 64, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 55, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 56, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 64, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 55, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 56, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, 64, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000000LL, FP_INT_TONEAREST, UINT_MAX, 0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 54, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 55, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, 64, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_UPWARD, UINT_MAX, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 54, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 55, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, 64, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_DOWNWARD, UINT_MAX, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 54, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 55, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, 64, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 54, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 55, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, 64, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 53, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 54, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 55, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, 64, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000000LL, FP_INT_TONEAREST, UINT_MAX, -0x20000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 55, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 56, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, 64, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_UPWARD, UINT_MAX, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 55, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 56, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, 64, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_DOWNWARD, UINT_MAX, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 55, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 56, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, 64, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 55, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 56, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 64, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 55, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 56, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, 64, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x20000000000001LL, FP_INT_TONEAREST, UINT_MAX, 0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 55, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 56, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, 64, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_UPWARD, UINT_MAX, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 55, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 56, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, 64, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_DOWNWARD, UINT_MAX, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 55, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 56, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, 64, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 55, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 56, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, 64, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 54, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 55, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 56, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, 64, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x20000000000001LL, FP_INT_TONEAREST, UINT_MAX, -0x20000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, 63, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, 64, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_UPWARD, UINT_MAX, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, 63, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, 64, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 63, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 64, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 63, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, 63, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, 64, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, 0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, 63, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, 64, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_UPWARD, UINT_MAX, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, 63, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, 64, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 63, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, 64, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 63, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, 63, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, 64, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, -0x3fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_UPWARD, 64, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_UPWARD, UINT_MAX, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_DOWNWARD, 64, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_DOWNWARD, UINT_MAX, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TOWARDZERO, 64, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 64, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEAREST, 64, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000000LL, FP_INT_TONEAREST, UINT_MAX, 0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, 63, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, 64, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_UPWARD, UINT_MAX, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, 63, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, 64, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_DOWNWARD, UINT_MAX, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, 63, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, 64, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TOWARDZERO, UINT_MAX, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 63, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, 64, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, 62, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, 63, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, 64, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000000LL, FP_INT_TONEAREST, UINT_MAX, -0x4000000000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_UPWARD, 64, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_UPWARD, UINT_MAX, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_DOWNWARD, 64, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_DOWNWARD, UINT_MAX, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TOWARDZERO, 64, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 64, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEAREST, 64, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4000000000000001LL, FP_INT_TONEAREST, UINT_MAX, 0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_UPWARD, 64, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_UPWARD, UINT_MAX, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_DOWNWARD, 64, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_DOWNWARD, UINT_MAX, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TOWARDZERO, 64, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TOWARDZERO, UINT_MAX, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, 64, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEAREST, 64, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4000000000000001LL, FP_INT_TONEAREST, UINT_MAX, -0x4000000000000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_UPWARD, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_UPWARD, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_DOWNWARD, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEAREST, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_UPWARD, 64, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_UPWARD, UINT_MAX, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_DOWNWARD, 64, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_DOWNWARD, UINT_MAX, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TOWARDZERO, 64, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TOWARDZERO, UINT_MAX, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, 64, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEAREST, 64, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffffffffffffLL, FP_INT_TONEAREST, UINT_MAX, -0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000000ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_UPWARD, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_UPWARD, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_DOWNWARD, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_DOWNWARD, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TOWARDZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TOWARDZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEARESTFROMZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEAREST, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -INT64_MAX-1, FP_INT_TONEAREST, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000001ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffffULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_UPWARD, 64, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_UPWARD, UINT_MAX, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_DOWNWARD, 64, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_DOWNWARD, UINT_MAX, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TOWARDZERO, 64, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 64, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEAREST, 64, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffff8000000000LL, FP_INT_TONEAREST, UINT_MAX, 0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_UPWARD, 64, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_UPWARD, UINT_MAX, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_DOWNWARD, 64, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_DOWNWARD, UINT_MAX, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TOWARDZERO, 64, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TOWARDZERO, UINT_MAX, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, 64, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEAREST, 64, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffff8000000000LL, FP_INT_TONEAREST, UINT_MAX, -0x7fffff8000000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000010000000000ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffff0000000000ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_UPWARD, 64, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_UPWARD, UINT_MAX, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 64, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_DOWNWARD, UINT_MAX, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 64, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, UINT_MAX, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 64, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEAREST, 64, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7ffffffffffffc00LL, FP_INT_TONEAREST, UINT_MAX, 0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_UPWARD, 64, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_UPWARD, UINT_MAX, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_DOWNWARD, 64, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_DOWNWARD, UINT_MAX, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, 64, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TOWARDZERO, UINT_MAX, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, 64, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEAREST, 64, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7ffffffffffffc00LL, FP_INT_TONEAREST, UINT_MAX, -0x7ffffffffffffc00LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x8000000000000800ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffffffffffff800ULL, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif

    /* Inputs with absolute value between 0 and 1.  */
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_subnorm_value, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_subnorm_value, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, min_value, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_value, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_value, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -min_value, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.25, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.25, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.25, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.25, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.499, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.499, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.499, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.499, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 106
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 113
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.5, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.5, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.5, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEAREST, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEAREST, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEAREST, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.5, FP_INT_TONEAREST, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 106
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 113
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.501, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.501, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.501, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.501, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.75, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.75, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.75, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.75, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0.9, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.9, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.9, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0.9, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 106
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 113
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_UPWARD, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 1, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 2, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, 64, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif

    /* Various non-integer inputs.  */
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.25, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.25, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.5, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.5, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 1.75, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -1.75, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.25, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.25, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.5, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.5, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 2.75, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -2.75, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.25, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.25, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.5, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.5, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 3.75, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -3.75, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_UPWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_DOWNWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TOWARDZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.25, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_UPWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_DOWNWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TOWARDZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.25, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_UPWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_DOWNWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TOWARDZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.5, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_UPWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_DOWNWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TOWARDZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.5, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_UPWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_DOWNWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TOWARDZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 4.75, FP_INT_TONEAREST, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_UPWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_DOWNWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TOWARDZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -4.75, FP_INT_TONEAREST, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_UPWARD, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_DOWNWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TOWARDZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.25, FP_INT_TONEAREST, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_UPWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_DOWNWARD, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TOWARDZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEARESTFROMZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.25, FP_INT_TONEAREST, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_UPWARD, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_DOWNWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TOWARDZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.5, FP_INT_TONEAREST, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_UPWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_DOWNWARD, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TOWARDZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEARESTFROMZERO, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.5, FP_INT_TONEAREST, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_UPWARD, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_DOWNWARD, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 4, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 5, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, 64, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TOWARDZERO, UINT_MAX, 5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 4, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 5, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, 64, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 5.75, FP_INT_TONEAREST, UINT_MAX, 6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_UPWARD, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_DOWNWARD, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 4, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 5, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, 64, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TOWARDZERO, UINT_MAX, -5, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEARESTFROMZERO, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 4, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 5, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, 64, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -5.75, FP_INT_TONEAREST, UINT_MAX, -6, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),

    /* 1, 1.5, 2, 2.5, 3, 3.5 +/- 1ulp.  */
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffp1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffp1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000002p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000002p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffff8p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffff8p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.000008p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.000008p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffcp0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffcp0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.000004p0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.000004p0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffff8p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffff8p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.000008p-1, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.000008p-1, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.fffffffffffff8p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.fffffffffffff8p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000001p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000001p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.ffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.ffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000004p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000004p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffep0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffep0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000002p0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000002p0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.ffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.ffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000004p-1, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000004p-1, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffp1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffp1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000002p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000002p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffcp0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffcp0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000004p0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000004p0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffff8p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000008p-1, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 106
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffcp1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.000000000000000000000000008p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.000000000000000000000000008p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.00000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.ffffffffffffffffffffffffffp0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.00000000000000000000000001p0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.00000000000000000000000001p0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.00000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 113
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p0, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep-1, FP_INT_TONEAREST, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 2, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 3, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 64, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, UINT_MAX, 1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_UPWARD, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_DOWNWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 1, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 2, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, 64, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TOWARDZERO, UINT_MAX, -1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x0.ffffffffffffffffffffffffffff8p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1.0000000000000000000000000001p1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x4.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x5.0000000000000000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 3, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 4, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 64, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_UPWARD, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_DOWNWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 2, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 3, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, 64, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -2, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x2.fffffffffffffffffffffffffffep0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3.0000000000000000000000000002p0, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x6.fffffffffffffffffffffffffffcp-1, FP_INT_TONEAREST, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 3, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 4, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 64, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, 3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 3, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 4, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 5, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 64, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, 4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_UPWARD, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_DOWNWARD, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 3, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 4, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, 64, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TOWARDZERO, UINT_MAX, -3, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEARESTFROMZERO, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 2, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 3, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 4, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, 64, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7.0000000000000000000000000004p-1, FP_INT_TONEAREST, UINT_MAX, -4, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif

    /* 31, 32, 33 fractional bits.  */
#if MANT_DIG >= 53
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_UPWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_UPWARD, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_UPWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_UPWARD, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_UPWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_DOWNWARD, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200000.80000002p0, FP_INT_TONEAREST, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_UPWARD, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_DOWNWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200000.80000002p0, FP_INT_TONEAREST, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_UPWARD, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_UPWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_UPWARD, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_UPWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_UPWARD, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_DOWNWARD, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 23, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 24, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, 64, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, 0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 23, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 24, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, 64, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x200001.80000002p0, FP_INT_TONEAREST, UINT_MAX, 0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_UPWARD, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_DOWNWARD, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 23, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 24, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, 64, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, -0x200001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 23, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 24, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, 64, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x200001.80000002p0, FP_INT_TONEAREST, UINT_MAX, -0x200002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_UPWARD, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 23, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 24, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 64, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 23, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 24, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 25, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, 64, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x3fffff.fffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_UPWARD, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 23, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 24, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, 64, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x3fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 23, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 24, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, 64, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x3fffff.fffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x400000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_UPWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_UPWARD, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_UPWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_UPWARD, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_UPWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000.80000001p0, FP_INT_TONEAREST, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_UPWARD, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_DOWNWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000.80000001p0, FP_INT_TONEAREST, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_UPWARD, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_UPWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_UPWARD, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_UPWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_UPWARD, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_DOWNWARD, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 22, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 23, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, 64, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 22, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 23, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, 64, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100001.80000001p0, FP_INT_TONEAREST, UINT_MAX, 0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_UPWARD, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_DOWNWARD, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 22, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 23, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, 64, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 22, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 23, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, 64, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100001.80000001p0, FP_INT_TONEAREST, UINT_MAX, -0x100002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_UPWARD, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 22, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 23, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 64, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 22, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 23, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 64, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 22, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 23, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 24, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, 64, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1fffff.ffffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 22, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 23, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, 64, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_UPWARD, UINT_MAX, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 22, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 23, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, 64, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0x1fffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 22, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 23, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, 64, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1fffff.ffffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x200000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_UPWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_UPWARD, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_UPWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 21, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 22, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, 64, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000.800000008p0, FP_INT_TONEAREST, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_UPWARD, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_DOWNWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 19, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 20, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 21, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, 64, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000.800000008p0, FP_INT_TONEAREST, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_UPWARD, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_UPWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_UPWARD, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_DOWNWARD, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 21, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 22, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, 64, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 21, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 22, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, 64, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80001.800000008p0, FP_INT_TONEAREST, UINT_MAX, 0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_UPWARD, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_DOWNWARD, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 21, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 22, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, 64, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80001, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 21, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 22, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, 64, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80001.800000008p0, FP_INT_TONEAREST, UINT_MAX, -0x80002, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 21, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 22, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 64, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 21, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 22, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 64, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 21, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 22, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 23, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, 64, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xfffff.ffffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 21, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 22, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, 64, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_UPWARD, UINT_MAX, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 21, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 22, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, 64, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0xfffff, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 20, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 21, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 22, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, 64, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xfffff.ffffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x100000, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_UPWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_UPWARD, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_UPWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_UPWARD, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_UPWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000000.80000002p0, FP_INT_TONEAREST, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_UPWARD, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_DOWNWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000000.80000002p0, FP_INT_TONEAREST, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_UPWARD, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_UPWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.7ffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_UPWARD, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_UPWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_UPWARD, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_DOWNWARD, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 34, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 35, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, 64, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, 0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 34, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 35, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, 64, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x100000001.80000002p0, FP_INT_TONEAREST, UINT_MAX, 0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_UPWARD, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_DOWNWARD, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 34, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 35, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, 64, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TOWARDZERO, UINT_MAX, -0x100000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 34, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 35, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, 64, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x100000001.80000002p0, FP_INT_TONEAREST, UINT_MAX, -0x100000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 34, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 35, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 36, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, 64, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_UPWARD, UINT_MAX, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 34, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 35, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 64, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, UINT_MAX, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 34, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 35, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 64, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, UINT_MAX, 0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 34, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 35, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 36, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 64, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 34, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 35, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 36, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 64, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x1ffffffff.fffffffep0, FP_INT_TONEAREST, UINT_MAX, 0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 34, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 35, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, 64, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_UPWARD, UINT_MAX, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 34, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 35, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, 64, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_DOWNWARD, UINT_MAX, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 34, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 35, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, 64, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TOWARDZERO, UINT_MAX, -0x1ffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 34, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 35, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, 64, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 34, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 35, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, 64, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x1ffffffff.fffffffep0, FP_INT_TONEAREST, UINT_MAX, -0x200000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_UPWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_UPWARD, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_UPWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_UPWARD, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_UPWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000000.80000001p0, FP_INT_TONEAREST, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_UPWARD, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_DOWNWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000000.80000001p0, FP_INT_TONEAREST, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_UPWARD, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_UPWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.7fffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_UPWARD, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_UPWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_UPWARD, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_DOWNWARD, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 33, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 34, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, 64, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, 0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 33, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 34, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, 64, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x80000001.80000001p0, FP_INT_TONEAREST, UINT_MAX, 0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_UPWARD, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_DOWNWARD, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 33, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 34, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, 64, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TOWARDZERO, UINT_MAX, -0x80000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 33, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 34, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, 64, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x80000001.80000001p0, FP_INT_TONEAREST, UINT_MAX, -0x80000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_UPWARD, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 33, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 34, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 64, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_DOWNWARD, UINT_MAX, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 33, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 34, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 64, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, 0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 33, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 34, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 35, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, 64, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0xffffffff.ffffffffp0, FP_INT_TONEAREST, UINT_MAX, 0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 33, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 34, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, 64, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_UPWARD, UINT_MAX, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_DOWNWARD, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 33, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 34, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, 64, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TOWARDZERO, UINT_MAX, -0xffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 33, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 34, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, 64, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0xffffffff.ffffffffp0, FP_INT_TONEAREST, UINT_MAX, -0x100000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_UPWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.8p0, FP_INT_TONEAREST, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_UPWARD, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.8p0, FP_INT_TONEAREST, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_UPWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 32, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 33, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, 64, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000000.800000008p0, FP_INT_TONEAREST, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_UPWARD, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 30, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 31, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 32, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, 64, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000000.800000008p0, FP_INT_TONEAREST, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_UPWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.7fffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_UPWARD, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.8p0, FP_INT_TONEAREST, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_UPWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.8p0, FP_INT_TONEAREST, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_UPWARD, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_DOWNWARD, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 32, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 33, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, 64, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, 0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 32, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 33, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, 64, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x40000001.800000008p0, FP_INT_TONEAREST, UINT_MAX, 0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_UPWARD, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_DOWNWARD, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 32, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 33, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, 64, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TOWARDZERO, UINT_MAX, -0x40000001LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 32, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 33, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, 64, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x40000001.800000008p0, FP_INT_TONEAREST, UINT_MAX, -0x40000002LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_UPWARD, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 32, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 33, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 64, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 32, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 33, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 64, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 32, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 33, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 34, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 64, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, UINT_MAX, 0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 32, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 33, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, 64, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_UPWARD, UINT_MAX, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_DOWNWARD, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 32, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 33, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, 64, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TOWARDZERO, UINT_MAX, -0x7fffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 31, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 32, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 33, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, 64, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x7fffffff.ffffffff8p0, FP_INT_TONEAREST, UINT_MAX, -0x80000000LL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
#endif

    /* Tests where overflow depends on the rounding mode.  */
#if MANT_DIG >= 64
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_DOWNWARD, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_DOWNWARD, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TOWARDZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.8p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif
#if MANT_DIG >= 72
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_DOWNWARD, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_DOWNWARD, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TOWARDZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEAREST, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.7fp0, FP_INT_TONEAREST, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_UPWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_DOWNWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_DOWNWARD, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_DOWNWARD, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TOWARDZERO, 64, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TOWARDZERO, UINT_MAX, 0x7fffffffffffffffLL, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0x7fffffffffffffff.81p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_UPWARD, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.7fp0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_UPWARD, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEAREST, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.8p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_UPWARD, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_UPWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TOWARDZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TOWARDZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEAREST, 65, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, 0xffffffffffffffff.81p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_UPWARD, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_UPWARD, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TOWARDZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TOWARDZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEARESTFROMZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEARESTFROMZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEARESTFROMZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEAREST, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.7fp0, FP_INT_TONEAREST, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_UPWARD, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_UPWARD, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TOWARDZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TOWARDZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEAREST, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEAREST, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.8p0, FP_INT_TONEAREST, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_UPWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_UPWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_UPWARD, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_UPWARD, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_UPWARD, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_DOWNWARD, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_DOWNWARD, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_DOWNWARD, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_DOWNWARD, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TOWARDZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TOWARDZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TOWARDZERO, 63, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TOWARDZERO, 64, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TOWARDZERO, UINT_MAX, -INT64_MAX-1, NO_INEXACT_EXCEPTION|ERRNO_UNCHANGED),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEARESTFROMZERO, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEARESTFROMZERO, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEARESTFROMZERO, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEARESTFROMZERO, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEAREST, 0, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEAREST, 1, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEAREST, 64, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
    TEST_fiu_M (fromfp, -0x8000000000000000.81p0, FP_INT_TONEAREST, UINT_MAX, IGNORE, NO_INEXACT_EXCEPTION|INVALID_EXCEPTION|ERRNO_EDOM),
#endif
  };

static void
fromfp_test (void)
{
  ALL_RM_TEST (fromfp, 1, fromfp_test_data, RUN_TEST_LOOP_fiu_M, END);
}

static void
do_test (void)
{
  fromfp_test ();
}

/*
 * Local Variables:
 * mode:c
 * End:
 */
